@@ -23,32 +23,32 @@
  * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
-$string['apiv1migration_message_content'] = 'As part of the recent Moodle upgrade to 2.6, the Box.net portfolio plugin has been disabled. To re-enable it, you need to re-configure it as described in the documentation {$a->docsurl}.';
-$string['apiv1migration_message_small'] = 'This plugin has been disabled, as it requires configuration as described in the documentation Box.net APIv1 migration.';
-$string['apiv1migration_message_subject'] = 'Important information regarding Box.net portfolio plugin';
+$string['apiv1migration_message_content'] = 'As part of the recent Moodle upgrade to 2.6, the Box portfolio plugin has been disabled. To re-enable it, you need to re-configure it as described in the documentation {$a->docsurl}.';
+$string['apiv1migration_message_small'] = 'This plugin has been disabled, as it requires configuration as described in the documentation Box APIv1 migration.';
+$string['apiv1migration_message_subject'] = 'Important information regarding Box portfolio plugin';
 $string['clientid'] = 'Client ID';
 $string['clientsecret'] = 'Client secret';
 $string['existingfolder'] = 'Existing folder to put file(s) into';
 $string['folderclash'] = 'The folder you asked to create already exists!';
-$string['foldercreatefailed'] = 'Failed to create your target folder on box.net';
-$string['folderlistfailed'] = 'Failed to retrieve a folder listing from box.net';
+$string['foldercreatefailed'] = 'Failed to create your target folder on Box';
+$string['folderlistfailed'] = 'Failed to retrieve a folder listing from Box';
 $string['missinghttps'] = 'HTTPS required';
-$string['missinghttps_help'] = 'Box.net will only work with an HTTPS enabled website.';
+$string['missinghttps_help'] = 'Box will only work with an HTTPS enabled website.';
 $string['missingoauthkeys'] = 'Missing client ID and secret';
-$string['missingoauthkeys_help'] = 'There is no client ID or secret configured for this plugin. You can get one of these from Box.net development page.';
+$string['missingoauthkeys_help'] = 'There is no client ID or secret configured for this plugin. You can get one of these from Box development page.';
 $string['newfolder'] = 'New folder to put file(s) into';
 $string['noauthtoken'] = 'Could not retrieve an authentication token for use in this session';
 $string['notarget'] = 'You must specify either an existing folder or a new folder to upload into';
-$string['noticket'] = 'Could not retrieve a ticket from box.net to begin the authentication session';
-$string['password'] = 'Your box.net password (will not be stored)';
-$string['pluginname'] = 'Box.net';
-$string['sendfailed'] = 'Failed to send content to box.net: {$a}';
+$string['noticket'] = 'Could not retrieve a ticket from Box to begin the authentication session';
+$string['password'] = 'Your Box password (will not be stored)';
+$string['pluginname'] = 'Box';
+$string['sendfailed'] = 'Failed to send content to Box: {$a}';
 $string['setupinfo'] = 'Setup instructions';
-$string['setupinfodetails'] = 'To obtain a client ID and secret, log in to Box.net and visit their <a href="{$a->servicesurl}">developers page</a>. Follow \'Create new application\' and create new application for your Moodle site. The client ID ans secret are displayed in \'OAuth2 parameters\' section of the application edit form. Optionally, you can also provide other information about your Moodle site.';
+$string['setupinfodetails'] = 'To obtain a client ID and secret, log in to Box and visit the <a href="{$a->servicesurl}">Box developers page</a>. Follow \'Create new application\' and create a new application for your Moodle site. The client ID and secret are displayed in the \'OAuth2 parameters\' section of the application edit form. Optionally, you can also provide other information about your Moodle site.';
 $string['sharedfolder'] = 'Shared';
 $string['sharefile'] = 'Share this file?';
 $string['sharefolder'] = 'Share this new folder?';
 $string['targetfolder'] = 'Target folder';
 $string['tobecreated'] = 'To be created';
-$string['username'] = 'Your box.net username (will not be stored)';
-$string['warninghttps'] = 'Box.net requires your website to be using HTTPS in order for the portfolio to work.';
+$string['username'] = 'Your Box username (will not be stored)';
+$string['warninghttps'] = 'Box requires your website to be using HTTPS in order for the portfolio to work.';
